from signalsdb.helpers import compile_re


def test_compile_glob():
    assert compile_re('\w*').match('4ny_tExt')
    assert compile_re('TEST*').match('testa')
    assert compile_re('ignores_case').match('IGNORES_CASE')



"""
    signalsdb.db
    ~~~~~~~~~~~~

    Dynamically initialized global signals database
    with explanations and default actions included.
"""


import signal


NAMES = {
    'SIGABRT': ('core', 'abort program (formerly SIGIOT)'),
    'SIGALRM': ('kill', 'real-time timer expired'),
    'SIGBUS': ('core', 'bus error'),
    'SIGCHLD': ('ignore', 'child status has changed'),
    'SIGCONT': ('ignore', 'continue after stop'),
    'SIGEMT': ('core', 'emulate instruction executed'),
    'SIGFPE': ('core', 'floating-point exception'),
    'SIGHUP': ('kill', 'terminal line hangup'),
    'SIGILL': ('core', 'illegal instruction'),
    'SIGINFO': ('ignore', 'status request from keyboard'),
    'SIGINT': ('kill', 'interrupt program'),
    'SIGIO': ('ignore', 'I/O is possible on a descriptor'),
    'SIGKILL': ('kill', 'kill program'),
    'SIGLIBRT': ('kill', 'real-time library interrupt'),
    'SIGPIPE': ('kill', 'write on a pipe with no readers'),
    'SIGPROF': ('kill', 'profiling timer alarm'),
    'SIGQUIT': ('core', 'quit program'),
    'SIGSEGV': ('core', 'invalid memory reference'),
    'SIGSTOP': ('stop', 'stop (cannot be caught or ignored)'),
    'SIGSYS': ('core', 'non-existent system call invoked'),
    'SIGTERM': ('kill', 'software termination signal'),
    'SIGTHR': ('kill', 'thread interrupt'),
    'SIGTRAP': ('core', 'trace trap'),
    'SIGTSTP': ('stop', 'stop signal generated from keyboard'),
    'SIGTTIN': ('stop', 'background read attempted from control terminal'),
    'SIGTTOU': ('stop', 'background write attempted to control terminal'),
    'SIGURG': ('ignore', 'urgent condition present on socket'),
    'SIGUSR1': ('kill', 'user defined signal 1'),
    'SIGUSR2': ('kill', 'user defined signal 2'),
    'SIGVTALRM': ('kill', 'virtual time alarm'),
    'SIGWINCH': ('ignore', 'Window size change'),
    'SIGXCPU': ('kill', 'cpu time limit exceeded'),
    'SIGXFSZ': ('kill', 'file size limit exceeded')
}


SIGNALS = {
    getattr(signal, k): (k,) + NAMES[k] for k in NAMES if
    hasattr(signal, k)
}



"""
    signalsdb.helpers
    ~~~~~~~~~~~~~~~~~

    Defines utility functions.
"""


import re


def compile_re(pattern):
    """
    Convert a *pattern* into a regex object compiled
    with the IGNORECASE flag.

    :param pattern: A regex (string).
    """
    return re.compile(pattern, re.IGNORECASE)


class NoSuchSignal(KeyError):
    """
    The given signal wasn't found in the DB.
    """
    pass



"""
    signalsdb.api
    ~~~~~~~~~~~~~

    Exports the public API.
"""


from signalsdb.db import SIGNALS
from signalsdb.helpers import compile_re, NoSuchSignal

__all__ = ('explain', 'search')


def explain(code, signals=SIGNALS):
    """
    Explain what a given *code* does, including it's
    signal name.

    :param code: An integer signal.
    :param signals: A database of signals.
    """
    if code not in signals:
        raise NoSuchSignal(code)
    signal, action, description = signals[code]
    return {
        'id': code,
        'signal': signal,
        'action': action,
        'description': description,
        }


def search(signal='', action='', signals=SIGNALS):
    """
    Search the signals DB for signal named *signal*,
    and which action matches *action* in a case
    insensitive way.

    :param signal: Regex for signal name.
    :param action: Regex for default action.
    :param signals: Database of signals.
    """
    signal = compile_re(signal).match
    action = compile_re(action).match
    res = []
    for code in signals:
        sig, act, _ = signals[code]
        if signal(sig) and action(act):
            res.append(explain(code, signals=signals))
    return res



from pytest import mark, raises
from signalsdb.api import explain, search, NoSuchSignal


SIGNALS = {
    13: ('SIGNAL', 'Action', 'Desc'),
    14: ('SIGNAT', 'Kill', 'Desc'),
}


def test_explain():
    assert explain(13, signals=SIGNALS) == {
            'action': 'Action',
            'description': 'Desc',
            'id': 13,
            'signal': 'SIGNAL',
        }


@mark.parametrize('signal,action', [
    ('signal', '(ACTION|Kill)'),
    ('SIGNA*', 'action'),
])
def test_search(signal, action):
    assert search(signal=signal,
                  action=action,
                  signals=SIGNALS) == [explain(13, signals=SIGNALS)]


def test_explain_fake():
    with raises(NoSuchSignal):
        explain(0, signals=SIGNALS)



"""
    signalsdb
    ~~~~~~~~~

    Python library providing a simple way to query a
    curated database of Unix signals.

    :copyright: (c) 2015 Eeo Jun
    :license: MIT, see LICENSE for details.
"""

__version__ = '0.1'



from distutils.core import setup


setup(
    name='signalsdb',
    version='0.1.1',
    url='https://github.com/eugene-eeo/signalsdb',
    author='Eeo Jun',
    author_email='packwolf58@gmail.com',
    license='MIT',
    packages=['signalsdb'],
    description='Query Unix signals',
    long_description=open('README.rst').read(),
    keywords=['signals', 'unix', 'db'],
)
